set stop 0

proc stopDraw {}\
{
	global stop
	set stop 1
}

proc drawAxis {{dash 0}}\
{
	global axis
	set c .res.can
	$c create line $axis(xa) $axis(ya) $axis(x0) $axis(y0) \
		-width 2 -tags axis
	$c create text [expr $axis(xa)+5] [expr $axis(ya)+5] -text X -tags axis
	$c create line $axis(xb) $axis(yb) $axis(x0) $axis(y0) \
		-width 2 -tags axis
	$c create text [expr $axis(xb)+5] [expr $axis(yb) +5] -text Y -tags axis
	$c create line $axis(xc) $axis(yc) $axis(x0) $axis(y0) \
		-width 2 -tags axis
	$c create text [expr $axis(xc)+5] [expr $axis(yc)+5] -text Z -tags axis
	if {$dash == 1}\
	{
		$c create line [expr 2*$axis(x0) - $axis(xa)] \
			[expr 2*$axis(y0) -$axis(ya)] \
			$axis(x0) $axis(y0) -tags axis -stipple gray50
		$c create line [expr 2*$axis(x0) - $axis(xb)] \
			[expr 2*$axis(y0) -$axis(yb)] \
			$axis(x0) $axis(y0) -tags axis -stipple gray50
		$c create line [expr 2*$axis(x0) - $axis(xc)] \
			[expr 2*$axis(y0) -$axis(yc)] \
			$axis(x0) $axis(y0) -tags axis -stipple gray50
	}
	$c configure -scrollregion [$c bbox all]
}

proc drawPixel {c x y z}\
{
	global axis colors zoom
	if {$x > 0}\
	{	set cx $colors(cxp)
	}\
	else\
	{	set cx $colors(cxn)
	}
	if {$y > 0}\
	{	set cy $colors(cyp)
	}\
	else\
	{	set cy $colors(cyn)
	}
	if {$z > 0}\
	{	set cz $colors(czp)
	}\
	else\
	{	set cz $colors(czn)
	}
	set na [expr sqrt(pow($axis(xa)-$axis(x0),2) + \
		pow($axis(ya)-$axis(y0),2))]
	if {$na == 0}\
	{	set na 0.00001
	}
	set nb [expr sqrt(pow($axis(xb)-$axis(x0),2) + \
		pow($axis(yb)-$axis(y0),2))]
	if {$nb == 0}\
	{	set nb 0.00001
	}
	set nc [expr sqrt(pow($axis(xc)-$axis(x0),2) + \
		pow($axis(yc)-$axis(y0),2))]
	if {$nc == 0}\
	{	set nc 0.00001
	}
	set k [expr $zoom*$x*($axis(xa)-$axis(x0))/$na +\
		$zoom*$y*($axis(xb)-$axis(x0))/$nb +\
		$zoom*$z*($axis(xc)-$axis(x0))/$nc + $axis(x0)]
	set l [expr $zoom*$x*($axis(ya)-$axis(y0))/$na +\
		$zoom*$y*($axis(yb)-$axis(y0))/$nb +\
		$zoom*$z*($axis(yc)-$axis(y0))/$nc +$axis(y0)]
	$c create oval [expr $k-1] [expr $l-1] [expr $k+1] [expr $l+1] \
		-fill #$cx$cy$cz -tags points -outline "" 
	update
}



proc setDefaults {}\
{
	setAxis 1 0 0 0 0 0 0 0 0 0
	setIncrements 1 1
	setBoundaries -10 10 -10 10
	setFunctions {$i} {$j} {$i + $j}
	setColors 33 66 99 DD FF AA
	setZoom 100
}

proc deletePoints {}\
{
	.res.can delete points
	update
}
	
proc setColors {cxn cxp cyn cyp czn czp}\
{
	global colors
	array set colors "cxn $cxn cxp $cxp cyn $cyn cyp $cyp czn $czn czp $czp"
}

proc setZoom {z}\
{
	global zoom
	set zoom [expr double($z)/100]
}

proc setAxis {def dash xa ya xb yb xc yc x0 y0}\
{
	global axis
	if {$def == 1}\
	{
		array set axis {xa 0 ya 480 xb 640 yb 240 xc 320 yc 0 \
			x0 320 y0 240}
	}\
	else\
	{
		array set axis "xa $xa ya $ya xb $xb yb $yb xc $xc yc $yc \
			x0 $x0 y0 $y0"
	}
	deleteAxis
	drawAxis $dash
}
	
proc deleteAxis {}\
{
	.res.can delete axis
	update
}

proc setIncrements {ci cj}\
{
	global incri incrj
	set incri $ci
	set incrj $cj
}

proc setBoundaries {sti spi stj spj}\
{
	global starti stopi startj stopj
	set starti $sti
	set stopi $spi
	set startj $stj
	set stopj $spj
}

proc setFunctions {fx fy fz}\
{	
	global fctx fcty fctz
	set fctx "$fx"
	set fcty "$fy"
	set fctz "$fz"
}

proc setFx {fx}\
{
	global fctx
	set fctx "$fx"
}

proc setFy {fy}\
{
	global fcty
	set fcty "$fy"
}

proc setFz {fz}\
{
	global fctz
	set fctz "$fz"
}

proc draw {}\
{
	global incri incrj starti stopi startj stopj fctx fcty fctz stop curi 
	set stop 0
	for {set i $starti} {$i <= $stopi} {set i [expr $i + $incri]}\
	{	
		.res.can configure -scrollregion [.res.can bbox all]
		for {set j $startj} {$j <= $stopj} {set j [expr $j + $incrj]}\
		{
			set ct [catch {drawPixel .res.can [expr $fctx] \
				[expr $fcty] [expr $fctz]
			} errmsg]
			if {$ct > 0}\
			{
				if {! [regexp ivide $errmsg]}\
				{
					myError $errmsg
					set stop 1
				}
			}	
			if {$stop == 1}\
			{	
				if {[.res.can cget -cursor] == "watch"}\
				{
					.res.can configure \
						-cursor top_left_arrow
				}
				set curi $i
				return
			}
		}	
	}
	if {[.res.can cget -cursor] == "watch"}\
	{
		.res.can configure -cursor top_left_arrow
	}
}

proc redraw {}\
{
	global incri incrj curi stopi startj stopj fctx fcty fctz stop
	set stop 0
	for {set i $curi} {$i <= $stopi} {set i [expr $i + $incri]}\
	{	
		.res.can configure -scrollregion [.res.can bbox all]
		for {set j $startj} {$j <= $stopj} {set j [expr $j + $incrj]}\
		{
			set ct [catch {drawPixel .res.can [expr $fctx] \
				[expr $fcty] [expr $fctz]
			} errmsg]
			if {$ct > 0}\
			{
				myError $errmsg
				set stop 1
			}	
			if {$stop == 1}\
			{	
				if {[.res.can cget -cursor] == "watch"}\
				{
					.res.can configure \
						-cursor top_left_arrow
				}
				set curi $i
				return
			}
		}	
	}
	if {[.res.can cget -cursor] == "watch"}\
	{
		.res.can configure -cursor top_left_arrow
	}
}
